/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of mst_rdb
 *
 * mst_rdb is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * mst_rdb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

/* rdb_rd_hdr.c */

/* for fileno */
#define _POSIX_C_SOURCE 1
#include <stdlib.h>
#undef _POSIX_C_SOURCE
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <suplib/io.h>

#include "mst_rdb.h"

/*

  PROCEDURE
    rdb_rd_hdr - Parse an /rdb file header prior to reading.

  DESCRIPTION
    Rdb_rd_hdr extracts column names from a stream connected to an
    /rdb data file.  It leaves the file pointer set to the first line after the
    header.  Lines preceding the header which are empty or begin with
    '#' are ignored.

  RETURNS
    This routine returns a pointer to a dynamically allocated rdbHeader
    structure (filled in with appropriate values).  It returns NULL if there are
    no header records.  Upon error it prints a message to stderr and exits.
    When the user is finished with these data structures, their memory
    should be freed with the rdb_free_hdr() function.

  AUTHOR
    Diab Jerius

*/
rdbHeader *
rdb_rd_hdr(
  FILE *fin			/* the stream to parse */
)
{
  unsigned long i, line = 0;
  char *buf, *p0, *p1;
  rdbHeader *hdr = NULL;
  void * fget_buf;

  tf_enter("rdb_rd_hdr");

  if ( NULL == ( fget_buf = fget_rec_new( 8192, 512 )) )
    tf_exit(ExitERR_alloc, "unable to allocate input buffer");
  
  /* perl rdb may have comments. ignore them as well as empty lines */
  while ( (buf = fget_rec(fin, fget_buf)) )
  {
    line++;
    /* move to first non-white-space character */
    while (isspace(*buf)) buf++;
    if (buf[0] && buf[0] != '#') break;
  }
  if (!buf)
  {
    fget_rec_delete( fget_buf );
    return NULL;
  }

  if ( (hdr = malloc(sizeof(rdbHeader))) == NULL )
    tf_exit(ExitERR_alloc, "unable to allocate rdb Header");

  hdr->buf = fget_buf;

  errno = 0;
  /* make a copy of the header line */
  if ( (hdr->names = malloc(strlen(buf) + 1)) == NULL )
    tf_exit(ExitERR_alloc, "unable to allocate column names");
  strcpy(hdr->names, buf);

  /* first line contains header information.  the number of columns is equal
     to the number of tabs plus one. replace tabs with EOS. */
  for (p0 = hdr->names, hdr->n = 1;
       (p1 = strchr(p0, '\t')) ;
       hdr->n++, p0 = p1 + 1)
    p1[0] = '\0';

  /* allocate pointers to names */
  if ( (hdr->col_name = malloc(sizeof(char *) * hdr->n)) == NULL )
    tf_exit(ExitERR_alloc, "unable to allocate column name pointers");

  p0 = hdr->names;
  for (i = 0; i < hdr->n ; i++)
  {
    char *p;

    /* remove leading and trailing spaces */
    while (*p0 && isspace(*p0)) p0++;
    if (! *p0)
      tf_exit(ExitERR_misc, "column %n has no name??", i+1);
    hdr->col_name[i] = p0;
    p0 += strlen(p0) + 1;
    p = p0 - 2;
    while (p > hdr->col_name[i] && isspace(*p)) *p-- = '\0';
  }

  /* skip next line */
  fget_rec(fin, hdr->buf);
  line++;

  hdr->line = line;

  /*
    record beginning of data, for seeks; if it's a pipe, can't do that
    so set start of data to -1
  */
  {
    struct stat stat_buf;
    /* find out if it's a pipe; if so it's an error */
    fstat( fileno(fin), &stat_buf );
    if( S_ISREG(stat_buf.st_mode) )
    {
      hdr->start = ftell(fin);
      hdr->can_rewind = 1;
    }
    else
    {
      hdr->start = -1;
      hdr->can_rewind = 0;
    }
  }


  tf_leave();
  return hdr;
}
